<?php
// $Id: content_retriever.admin.inc,v 1.1.2.6.2.4 2010/02/10 14:25:32 joachim Exp $
/**
 * @file
 * Admin page callbacks for the content retriever module.
 */

/**
 * form containing available admin settings
 * 
 * @return
 * array containing the form
 */
function content_retriever_admin_settings_form($form_state) {
  //build the checkbox fields from the current local content types
  if (variable_get('drupal_http_request_fails', FALSE) == TRUE) {
    drupal_set_message(t('Drupal is unable to make HTTP requests. Please reset the HTTP request status.'), 'error', FALSE);
    watchdog('integration', t('Drupal is unable to make HTTP requests. Please reset the HTTP request status.'), array(), WATCHDOG_CRITICAL);
  }
  $last_run = variable_get('content_retriever_last_run',format_date(time(), 'custom', 'Y-m-d H:i'));
  $last_run = format_date($last_run, 'custom', 'Y-m-d H:i');
  $form = array();
  $form['notice'] = array(
    '#type' => 'markup', 
    '#value' => '<p><span style="color:red; font-weight: bold">Important!</span> ' . t('After a failed HTTP request (e.g. web service goes down, endpoint URL is wrong, etc.) Drupal assumes it has no access to the Internet and prevents itself from trying to make HTTP requests again. You may use the "Reset HTTP request status" button below to reset Drupal and allow it to make HTTP requests again. Always try this if you are not receiving data and you don\'t know why.') . '</p>',
  );
  $form['endpoint'] = array(
	'#title' => t('Web service endpoint'),
	'#type' => 'textfield',
	'#required' => 'true',
	'#default_value' => variable_get('content_retriever_endpoint', 'http://cms.defaqto.com/services/xmlrpc'),
  );
  $form['api_key'] = array(
	'#title' => t('Web service API key'),
	'#type' => 'textfield',
	'#default_value' => variable_get('content_retriever_api_key', ''),
  );
  $form['username'] = array(
	'#title' => t('Web service User name'),
	'#type' => 'textfield',
	'#default_value' => variable_get('content_retriever_webservice_username', ''),
  );
  $form['password'] = array(
	'#title' => t('Web service Password'),
	'#type' => 'textfield',
	'#default_value' => variable_get('content_retriever_webservice_pass', ''),
  );
  $form['reset_timestamp'] = array(
    '#type' => 'textfield',
    '#title' => t('Reset the time the retriever was last run'),
    '#size' => 40,
    '#default_value' => '',
    '#description' => t('Use the Date Format: YYYY-MM-DD HH:MM. Leave blank to leave the time set to %time.', array(
      '%time' => $last_run,
     )),
  );
  $form['reporting'] = array(
    '#type' => 'checkbox',
    '#title' => 'On screen reporting messages',
    '#default_value' => variable_get('content_retriever_screen_messages', 0),
  ); 
  $form['submit'] = array(
    '#value' => 'Save settings',
	'#type' => 'submit',
  );
  $form['retrieve'] = array(
    '#value' => 'Retrieve content now',
    '#type' => 'submit',
    '#name' => 'retrieve',
    '#submit' => array('content_retriever_admin_retrieve_now_form_submit'),
  );
  $form['reset_status'] = array(
	'#value' => 'Reset HTTP request status',
	'#type' => 'submit',
	'#name' => 'reset_status',
	'#submit' => array('content_retriever_admin_clear_http_status_form_submit'),
  );
  
  return $form; 
}

/**
 * submit function fires the node retrieval function
 */
function content_retriever_admin_retrieve_now_form_submit($form, &$form_state) {
  $last_run = variable_get('content_retriever_last_run', (time() - (24 * 60 * 60)) );
  if (_content_retriever_save_nodes($last_run)) {
    drupal_set_message(t('Process complete.'));
    variable_set('content_retriever_last_run', time());
  }
}

/**
 * submit function sets the HTTP request variable back to FALSE if it has
 * inadvertently been made TRUE by a faulty web service/bad address
 */
function content_retriever_admin_clear_http_status_form_submit($form, &$form_state) {
  variable_set('drupal_http_request_fails', FALSE);
  drupal_set_message(t('HTTP request status reset.'));
}

/**
 * validate function to check user-entered data from the admin form
 */
function content_retriever_admin_settings_form_validate($form, &$form_state) {
  if (strpos($form_state['values']['endpoint'], 'http') != 0) {
    form_set_error('endpoint', t('The service endpoint must be a valid HTTP or HTTPS web address.'));
  }
  $reset = $form_state['values']['reset_timestamp'];
  if ($reset) {
    // get the current timestamp and convert user entered date to
    // timestamp to compare
    $now = time();
    $reset_timestamp = date_convert($reset,DATE_DATETIME, DATE_UNIX);
    
    // set form error if invalid date format
    if (!date_is_valid($reset, $type = DATE_DATETIME)) {
    	form_set_error('reset_timestamp', t('Please enter a valid date format.'));
    }
    elseif ($reset_timestamp > $now) {
    	form_set_error('reset_timestamp', t('Please enter a time in the past to reset the last run time to.'));
    }
  }
  
}

/**
 * submit function for 'Settings' form.
 */
function content_retriever_admin_settings_form_submit($form, &$form_state) {
  // save api key and endpoint variables
  variable_set('content_retriever_endpoint', $form_state['values']['endpoint']);
  variable_set('content_retriever_api_key', $form_state['values']['api_key']);
  variable_set('content_retriever_webservice_username', $form_state['values']['username']);
  variable_set('content_retriever_webservice_pass', $form_state['values']['password']);
  variable_set('content_retriever_screen_messages', $form_state['values']['reporting']);
  
  $time = $form_state['values']['reset_timestamp'];
  if ($time) {
    // convert date to unix timestamp
    $reset = date_convert($time,DATE_DATETIME, DATE_UNIX);
    variable_set('content_retriever_last_run', $reset);
  }
  
  drupal_set_message(t('Content Retriever Settings Saved.'));
}

/**
 * View name form. Page 1 of 3.
 */
function content_retriever_admin_view_form($form_state) {
  $form['view'] = array(
    '#title' => t('Name of the view to query on the distribution server'),
    '#type' => 'textfield',
    '#required' => 'true',
    '#default_value' => variable_get('content_retriever_viewname', 'content_distribution_queue'),
  );

  $form['submit'] = array(
    '#value' => t('Next'),
	  '#type' => 'submit',
  );

  return $form; 
}

/**
 * Submit handler for view name form.
 */
function content_retriever_admin_view_form_submit($form, &$form_state) {
  variable_set('content_retriever_viewname', $form_state['values']['view']);

  drupal_set_message(t('The view setting has been saved.'));
  
  // Advance to the next page.
  $form_state['redirect'] = 'admin/settings/retriever/view/display';  
}

/**
 * View display form. Page 2 of 3.
 */
function content_retriever_admin_view_display_form($form_state) {
  // Get the view object from the Distributor and extract the names
  // of its displays to use as options.
  $view_name = variable_get('content_retriever_viewname', 'content_distribution_queue');
  $view = content_retriever_xmlrpc('views.getViewObject', $view_name);
  // For some reason we have an array here not an object... @todo, but just a minor glitch really
  $options = array();
  if (empty($view)) {
  	drupal_set_message(t('Could not retrieve a view from the remote server'), 'error');
  }
  else {
    foreach ($view['display'] as $display_id => $display) {
      $options[$display_id] = $display['display_title'];
    }
  }
  
  $form['info'] = array(
	  '#type' => 'markup',
    '#value' => t('The following displays have been retrieved from the %view view.', array('%view' => $view_name)),
  );
  
  $form['view_display'] = array(
    '#title' => t('Display to query on the distribution server'),
    '#type' => 'radios',
    '#options' => $options,
    '#required' => 'true',
    '#default_value' => variable_get('content_retriever_view_displayname', 'default'),
  );

  $form['submit'] = array(
    '#value' => t('Next'),
	  '#type' => 'submit',
  );

  return $form; 
} 

/**
 * Submit handler for view display form.
 */
function content_retriever_admin_view_display_form_submit($form, &$form_state) {
  variable_set('content_retriever_view_displayname', $form_state['values']['view_display']);
  
  drupal_set_message(t('The display setting has been saved.'));
  
  // Advance to the next page.
  $form_state['redirect'] = 'admin/settings/retriever/view/arguments';  
}

/**
 * View arguments form. Page 3 of 3.
 *
 * @todo: we get error messages here if the user plays around and goes back to
 * change the view or display, as the form tries to fill in default values that 
 * don't match the new form. Add some checks and things perhaps.
 */
function content_retriever_admin_view_arguments_form($form_state) {
  $form = array();
  
  // Get the view object from the Distributor and extract the names
  // of its displays to use as options.
  $view_name = variable_get('content_retriever_viewname', 'content_distribution_queue');
  $view_display_name = variable_get('content_retriever_view_displayname', 'default');
  
  $view = content_retriever_xmlrpc('views.getViewObject', $view_name);
  // For some reason we have an array here not an object... @todo, but just a minor glitch really
  
  $options = array();
  if (empty($view)) {
  	drupal_set_message(t('Could not retrieve a view from the remote server'), 'error');
  	return $form;
  }

  // Get the list of the view's arguments, removing the final one for node
  // changed date, which is set automatically.
  // @todo, maybe: if the final argument is not the right one, warn the user
  // that the view is no good!
  $view_arguments = $view['display'][$view_display_name]['display_options']['arguments'];
  array_pop($view_arguments);

  // This comes to us fairly raw, as a list of the view's arguments with table 
  // and field names.
  // We want to give submodules the chance to turn this into a more usable UI, 
  // eg if there is a 'node type' argument, select types.
  // For each argument, there may be a module that can help process this, 
  // or they may not, in which case we show a standard textfield. 
  // We pass two things to a hook: the full list we got from the view,
  // and a results array constructed from the id keys and NULL values.
  // If a module can act on an argument, it should populate the results array
  // value with a form element. Modules should only act on a NULL value 
  // (ie, don't clobber what another module has done. 
  // We fill in remaining NULL values afterwards.

  $results = array_fill_keys(array_keys($view_arguments), NULL);
  
  // Invoke the hook on all modules. We don't use module_invoke_all so we can 
  // pass by reference.
  module_load_all_includes('content_retriever_arguments_process.inc');
  foreach (module_implements('content_retriever_views_arguments_process') as $module) {
    $function = $module . '_content_retriever_views_arguments_process';
    $function($view_arguments, $results);
  }
  
  $form['info'] = array(
	  '#type' => 'markup',
    '#value' => t("The following arguments have been retrieved from the %display display of the %view view. The final 'node updated' argument is not shown.", array(
      '%view'    => $view_name,
      '%display' => $view_display_name
    )),
  );
  

  $argument_values = variable_get('content_retriever_view_arguments', '');
  
  // Set #tree on a wrapper so we can easily collect all the values on submit.
  $form['view_arguments'] = array(
    '#tree' => TRUE,
  );
  
  foreach($view_arguments as $argument_id => $argument) {
    if (is_null($results[$argument_id])) {
      // This argument needs a default textfield widget.
      $form['view_arguments'][$argument_id] = array(
        '#title' => t('Value for argument %arg', array('%arg' => $argument_id)),
        '#type' => 'textfield',
        '#default_value' => $argument_values[$argument_id],
      );
    }
    else {
      // A hook invocation contributed a form element. Insert the default value.
      $form['view_arguments'][$argument_id] = $results[$argument_id];
      $form['view_arguments'][$argument_id]['#default_value'] = $argument_values[$argument_id];
    }
  }
  
  $form['submit'] = array(
    '#value' => 'Save settings',
	  '#type' => 'submit',
  );
  return $form; 
}

/**
 * Submit handler for view arguments form.
 */
function content_retriever_admin_view_arguments_form_submit($form, &$form_state) {
  // Filter out 0 values in checkboxes so they don't get saved and later passed
  // as arguments to the view.
  foreach ($form_state['values']['view_arguments'] as $id => $value) {
    if (is_array($value)) {
      $form_state['values']['view_arguments'][$id] = array_filter($value);
    }
  }
  
  // This stores the whole array of values, keyed by argument id.
  variable_set('content_retriever_view_arguments', $form_state['values']['view_arguments']);
  
  drupal_set_message(t('The argument settings have been saved.'));
}
