<?php
// $Id: distributor_service.inc,v 1.1.2.4.2.2 2010/02/09 13:55:18 joachim Exp $
/**
 * @author gtaylor
 * @file
 * Define services for content distribution.
 */


/**
 * Returns a specified node.
 *
 * @param $type
 *   String. The node Type.
 * 
 * @return
 *   Boolean. The node is set up to be distributed or not.
 */
function distributor_service_get_type($type) {
  $is_distributable = variable_get('distribute_content_type_' . $type, 0);
  if ($is_distributable !== 0) {
    return TRUE;	
  }
  else {
  	return FALSE;
  }
}

/**
 * Determine whether all content types are set up 
 * for distribution.
 *
 * @return
 *   Array. Array of content types with a flag if
 *   their nodes are distributable or not.
 */
function distributor_service_get_all_types() {
  $content_types = node_get_types();
  foreach ($content_types as $type) {
  	$is_distributable = variable_get('distribute_content_type_' . $type->type,0);
  	if ($is_distributable !== 0) {
  		$types[$type->type] = $type->type;
  	}
  }
  return $types;  
}


/**
 * Looks up the node id on the local server based on the node id which
 * is being deleted on the other site and deletes the corresponding node.
 * 
 * @param $nid
 * the node id to look up
 */
function distributor_service_delete_node($nid) {
  global $user;
  
  //we need to be a superuser to do this next stuff
  //this should be refactored so the webservice logs in
  //using the user service
  $temp_user = $user;
  $user = user_load(array('uid' => 1));
  
  // find the local node id.
  $result = db_query("SELECT nid FROM {content_retriever} WHERE original_nid = %d", $nid);
  $node_id = (int)db_result($result);
  
  //delete the node
  if ($node_id) {
    node_delete($node_id);
  }
  $user = $temp_user;  
  return $node_id;
}

/**
 * Services callback: return a view object.
 *
 * @param $view_name
 *   String. The view name.
 * @return
 *   Object. The view object.
 */
function distributor_service_get_view_object($view_name) {
  $view = views_get_view($view_name);
  
  return $view;
}
